@@ -23,15 +23,16 @@
 using System;
 using System.Collections;
 using System.Collections.Generic;
+using System.Globalization;
 
 namespace TagLib.Ogg
 {
-   public class XiphComment : TagLib.Tag, IEnumerable
+   public class XiphComment : TagLib.Tag, IEnumerable<string>
    {
       //////////////////////////////////////////////////////////////////////////
       // private properties
       //////////////////////////////////////////////////////////////////////////
-      private Dictionary<string, StringList> field_list;
+      private Dictionary<string, StringCollection> field_list;
       private string vendor_id;
       private string comment_field;
       
@@ -40,9 +41,8 @@ namespace TagLib.Ogg
       //////////////////////////////////////////////////////////////////////////
       public XiphComment () : base ()
       {
-         field_list = new Dictionary<string, StringList> ();
-         vendor_id = null;
-         comment_field = null;
+         field_list = new Dictionary<string, StringCollection> ();
+         comment_field = "DESCRIPTION";
       }
       
       public XiphComment (ByteVector data) : this ()
@@ -50,72 +50,63 @@ namespace TagLib.Ogg
          Parse (data);
       }
       
-      public StringList GetField (string key)
+      public override void Clear ()
       {
-         return (field_list.ContainsKey (key.ToUpper ())) ? field_list [key.ToUpper ()] : null;
+         field_list.Clear ();
       }
       
-      public void AddField (string key, string value, bool replace)
+      public string [] GetField (string key)
       {
-         if (replace)
-            RemoveField (key.ToUpper ());
-         
-         if (value != null && value.Trim () != "")
-         {
-            if (!field_list.ContainsKey (key.ToUpper ()))
-               field_list.Add (key.ToUpper (), new StringList ());
-         
-            field_list [key.ToUpper ()].Add (value);
-         }
+         key = key.ToUpper (CultureInfo.InvariantCulture);
+         return (field_list.ContainsKey (key)) ? field_list [key].ToArray () : new string [0];
       }
       
-      public void AddField (string key, string value)
+      public string GetFirstField (string key)
       {
-         AddField (key, value, true);
+         string [] values = GetField (key);
+         return (values.Length > 0) ? values [0] : null;
       }
       
-      public void AddFields (string key, string [] values, bool replace)
+      public void SetField (string key, uint number)
       {
-         if (replace)
-            RemoveField (key.ToUpper ());
-         
-         foreach (string s in values)
-            AddField (key, s, false);
-      }
-      
-      public void AddFields (string key, string [] values)
-      {
-         AddFields (key, values, true);
-      }
-      
-      public void RemoveField (string key, string value)
-      {
-         if (!field_list.ContainsKey (key.ToUpper ()))
-            return;
-         
-         StringList l = field_list [key.ToUpper ()];
-         
-         if (value == null)
-            l.Clear ();
+         if (number == 0)
+            RemoveField (key);
          else
-         {
-            int index;         
-            while ((index = l.IndexOf (value)) >=0)
-               l.RemoveAt (index);
-         }
+            SetField (key, number.ToString (CultureInfo.InvariantCulture));
+      }
+      
+      public void SetField (string key, params string [] values)
+      {
+         key = key.ToUpper (CultureInfo.InvariantCulture);
+         
+         StringCollection results = new StringCollection ();
+         
+         if (values != null)
+            foreach (string value in values)
+               if (value != null && value.Trim ().Length != 0)
+                  results.Add (value);
+         
+         if (results.Count == 0)
+            RemoveField (key);
+         else if (field_list.ContainsKey (key))
+            field_list [key] = results;
+         else
+            field_list.Add (key, results);
       }
       
       public void RemoveField (string key)
       {
-         RemoveField (key, null);
+         StringCollection values;
+         if (field_list.TryGetValue (key.ToUpper (CultureInfo.InvariantCulture), out values))
+            values.Clear ();
       }
       
-      public ByteVector Render (bool add_framing_bit)
+      public ByteVector Render (bool addFramingBit)
       {
          ByteVector data = new ByteVector ();
 
          // Add the vendor ID length and the vendor ID.  It's important to use the
-         // lenght of the data(String::UTF8) rather than the lenght of the the string
+         // length of the data(String::UTF8) rather than the lenght of the the string
          // since this is UTF8 text and there may be more characters in the data than
          // in the UTF16 string.
 
@@ -129,15 +120,15 @@ namespace TagLib.Ogg
          data.Add (ByteVector.FromUInt (FieldCount, false));
 
          // Iterate over the the field lists.  Our iterator returns a
-         // std::pair<String, StringList> where the first String is the field name and
-         // the StringList is the values associated with that field.
+         // std::pair<String, StringCollection> where the first String is the field name and
+         // the StringCollection is the values associated with that field.
 
-         foreach (KeyValuePair<string, StringList> de in field_list)
+         foreach (KeyValuePair<string, StringCollection> de in field_list)
          {
             // And now iterate over the values of the current list.
 
             string field_name = de.Key;
-            StringList values = de.Value;
+            StringCollection values = de.Value;
 
             foreach (string value in values)
             {
@@ -151,18 +142,18 @@ namespace TagLib.Ogg
          }
 
          // Append the "framing bit".
-         if (add_framing_bit)
+         if (addFramingBit)
             data.Add ((byte) 1);
 
          return data;
       }
       
-      public ByteVector Render ()
+      public IEnumerator<string> GetEnumerator()
       {
-         return Render (true);
+         return field_list.Keys.GetEnumerator();
       }
       
-      public IEnumerator GetEnumerator()
+      IEnumerator IEnumerable.GetEnumerator()
       {
          return field_list.Keys.GetEnumerator();
       }
@@ -170,11 +161,13 @@ namespace TagLib.Ogg
       //////////////////////////////////////////////////////////////////////////
       // public properties
       //////////////////////////////////////////////////////////////////////////
+      public override TagTypes TagTypes {get {return TagTypes.Xiph;}}
+      
       public override bool IsEmpty
       {
          get
          {
-            foreach (StringList l in field_list.Values)
+            foreach (StringCollection l in field_list.Values)
                if (!l.IsEmpty)
                   return false;
             
@@ -187,7 +180,7 @@ namespace TagLib.Ogg
          get
          {
             uint count = 0;
-            foreach (StringList l in field_list.Values)
+            foreach (StringCollection l in field_list.Values)
                count += (uint) l.Count;
             
             return count;
@@ -204,135 +197,82 @@ namespace TagLib.Ogg
 
       public override string Title
       {
-         get
-         {
-            StringList l = GetField ("TITLE");
-            return (l != null && l.Count != 0) ? l [0] : null;
-         }
-         set
-         {
-            AddField ("TITLE", value);
-         }
+         get {return GetFirstField ("TITLE");}
+         set {SetField ("TITLE", value);}
       }
       
       public override string [] AlbumArtists
       {
-         get
-         {
-            StringList l = GetField ("ARTIST");
-            return (l != null && l.Count != 0) ? l.ToArray () : new string [] {};
-         }
-         set
-         {
-            AddFields ("ARTIST", value);
-         }
+         get {return GetField ("ALBUMARTIST");}
+         set {SetField ("ALBUMARTIST", value);}
       }
       
       public override string [] Performers
       {
-         get
-         {
-            StringList l = GetField ("PERFORMER");
-            return (l != null && l.Count != 0) ? l.ToArray () : new string [] {};
-         }
-         set
-         {
-            AddFields ("PERFORMER", value);
-         }
+         get {return GetField ("ARTIST");}
+         set {SetField ("ARTIST", value);}
       }
       
       public override string [] Composers
       {
-         get
-         {
-            StringList l = GetField ("COMPOSER");
-            return (l != null && l.Count != 0) ? l.ToArray () : new string [] {};
-         }
-         set
-         {
-            AddFields ("COMPOSER", value);
-         }
+         get {return GetField ("COMPOSER");}
+         set {SetField ("COMPOSER", value);}
       }
 
       public override string Album
       {
-         get
-         {
-            StringList l = GetField ("ALBUM");
-            return (l != null && l.Count != 0) ? l [0] : null;
-         }
-         set
-         {
-            AddField ("ALBUM", value);
-         }
+         get {return GetFirstField ("ALBUM");}
+         set {SetField ("ALBUM", value);}
       }
 
       public override string Comment
       {
          get
          {
-            StringList l = GetField ("DESCRIPTION");
-            comment_field = "DESCRIPTION";
+            string value = GetFirstField (comment_field);
+            if (value != null || comment_field == "COMMENT")
+               return value;
             
-            if (l == null || l.Count == 0)
-            {
-               l = GetField ("COMMENT");
-               comment_field = "COMMENT";
-            }
-            
-            return (l != null && l.Count != 0) ? l [0] : null;
-         }
-         set
-         {
-            AddField (comment_field == null ? "DESCRIPTION" : comment_field, value);
+            comment_field = "COMMENT";
+            return GetFirstField (comment_field);
          }
+         set {SetField (comment_field, value);}
       }
       
       public override string [] Genres
       {
-         get
-         {
-            StringList l = GetField ("GENRE");
-            return (l != null && l.Count != 0) ? l.ToArray () : new string [] {};
-         }
-         set
-         {
-            AddFields ("GENRE", value);
-         }
+         get {return GetField ("GENRE");}
+         set {SetField ("GENRE", value);}
       }
       
       public override uint Year
       {
          get
          {
-            try
-            {
-               StringList l = GetField ("DATE");
-               return (l != null && l.Count != 0) ? UInt32.Parse (l [0].Substring (0, 4)) : 0;
-            }
-            catch {return 0;}
-         }
-         set
-         {
-            AddField ("DATE", value.ToString ());
+            string text = GetFirstField ("DATE");
+            uint value;
+            return (text != null && uint.TryParse (text.Length > 4 ? text.Substring (0, 4) : text, out value)) ? value : 0;
          }
+         set {SetField ("DATE", value);}
       }
       
       public override uint Track
       {
          get
          {
-            try
-            {
-               StringList l = GetField ("TRACKNUMBER");
-               return (l != null && l.Count != 0) ? UInt32.Parse (l [0].Split (new char [] {'/'}) [0]) : 0;
-            }
-            catch {return 0;}
+            string text = GetFirstField ("TRACKNUMBER");
+            string [] values;
+            uint value;
+            
+            if (text != null && (values = text.Split ('/')).Length > 0 && uint.TryParse (values [0], out value))
+               return value;
+            
+            return 0;
          }
          set
          {
-            TrackCount = TrackCount;
-            AddField ("TRACKNUMBER", value.ToString ());
+            SetField ("TRACKTOTAL", TrackCount);
+            SetField ("TRACKNUMBER", value);
          }
       }
       
@@ -340,38 +280,38 @@ namespace TagLib.Ogg
       {
          get
          {
-            try
-            {
-               StringList l = GetField ("TRACKTOTAL");
-               if (l != null && l.Count != 0)
-                  return UInt32.Parse (l [0]);
-               
-               l = GetField ("TRACKNUMBER");
-               return (l != null && l.Count != 0) ? UInt32.Parse (l [0].Split (new char [] {'/'}) [1]) : 0;
-            }
-            catch {return 0;}
-         }
-         set
-         {
-            AddField ("TRACKTOTAL", value.ToString ());
+            string text;
+            string [] values;
+            uint value;
+            
+            if ((text = GetFirstField ("TRACKTOTAL")) != null && uint.TryParse (text, out value))
+               return value;
+            
+            if ((text = GetFirstField ("TRACKNUMBER")) != null && (values = text.Split ('/')).Length > 1 && uint.TryParse (values [1], out value))
+               return value;
+            
+            return 0;
          }
+         set {SetField ("TRACKTOTAL", value);}
       }
       
       public override uint Disc
       {
          get
          {
-            try
-            {
-               StringList l = GetField ("DISCNUMBER");
-               return (l != null && l.Count != 0) ? UInt32.Parse (l [0].Split (new char [] {'/'}) [0]) : 0;
-            }
-            catch {return 0;}
+            string text = GetFirstField ("DISCNUMBER");
+            string [] values;
+            uint value;
+            
+            if (text != null && (values = text.Split ('/')).Length > 0 && uint.TryParse (values [0], out value))
+               return value;
+            
+            return 0;
          }
          set
          {
-            DiscCount = DiscCount;
-            AddField ("DISCNUMBER", value.ToString ());
+            SetField ("DISCTOTAL", TrackCount);
+            SetField ("DISCNUMBER", value);
          }
       }
       
@@ -379,23 +319,56 @@ namespace TagLib.Ogg
       {
          get
          {
-            try
-            {
-               StringList l = GetField ("DISCTOTAL");
-               if (l != null && l.Count != 0)
-                  return UInt32.Parse (l [0]);
-               
-               l = GetField ("DISCNUMBER");
-               return (l != null && l.Count != 0) ? UInt32.Parse (l [0].Split (new char [] {'/'}) [1]) : 0;
-            }
-            catch {return 0;}
-         }
-         set
-         {
-            AddField ("DISCTOTAL", value.ToString ());
+            string text;
+            string [] values;
+            uint value;
+            
+            if ((text = GetFirstField ("DISCTOTAL")) != null && uint.TryParse (text, out value))
+               return value;
+            
+            if ((text = GetFirstField ("DISCNUMBER")) != null && (values = text.Split ('/')).Length > 1 && uint.TryParse (values [1], out value))
+               return value;
+            
+            return 0;
          }
+         set {SetField ("DISCTOTAL", value);}
       }
       
+      public override string Lyrics
+      {
+         get {return GetFirstField ("LYRICS");}
+         set {SetField ("LYRICS", value);}
+      }
+      
+      public override string Copyright
+      {
+         get {return GetFirstField ("COPYRIGHT");}
+         set {SetField ("COPYRIGHT", value);}
+      }
+      
+      public override string Conductor
+      {
+         get {return GetFirstField ("CONDUCTOR");}
+         set {SetField ("CONDUCTOR", value);}
+      }
+      
+      public override string Grouping
+      {
+         get {return GetFirstField ("GROUPING");}
+         set {SetField ("GROUPING", value);}
+      }
+      
+      public override uint BeatsPerMinute
+      {
+         get
+         {
+            string text = GetFirstField ("TEMPO");
+            double value;
+            return (text != null && double.TryParse (text, out value)
+		&& value > 0) ? (uint) Math.Round (value) : 0;
+         }
+         set {SetField ("TEMPO", value);}
+      }
       
       //////////////////////////////////////////////////////////////////////////
       // protected methods
@@ -432,10 +405,13 @@ namespace TagLib.Ogg
 
             int comment_separator_position = comment.IndexOf ('=');
 
-            string key = comment.Substring (0, comment_separator_position);
+            string key = comment.Substring (0, comment_separator_position).ToUpper (CultureInfo.InvariantCulture);
             string value = comment.Substring (comment_separator_position + 1);
-
-            AddField (key, value, false);
+            
+            if (field_list.ContainsKey (key))
+               field_list [key].Add (value);
+            else
+               SetField (key, value);
          }
       }
    }
