@@ -66,8 +66,8 @@ namespace DAAP {
                                          "daap.songdiscnumber,daap.songdisabled,daap.songeqpreset," +
                                          "daap.songformat,daap.songgenre,daap.songdescription," +
                                          "daap.songsamplerate,daap.songsize,daap.songstarttime," +
-                                         "daap.songstoptime,daap.songtime,daap.songsongcount," +
-                                         "daap.songsongnumber,daap.songuserrating,daap.songyear," +
+                                         "daap.songstoptime,daap.songtime,daap.songtrackcount," +
+                                         "daap.songtracknumber,daap.songuserrating,daap.songyear," +
                                          "daap.songdatakind,daap.songdataurl,com.apple.itunes.norm-volume," +
                                          "com.apple.itunes.itms-songid,com.apple.itunes.itms-artistid," +
                                          "com.apple.itunes.itms-playlistid,com.apple.itunes.itms-composerid," +
